"""Functions used for timing or time calculations.

Implements: `date_range`, `ensure_section_duration`, `set_alarm`,
`clear_alarm`.

Some of the functions in here are duplicate as in the `context`
module because they fit better here. The functions in `context`
have been deprecated and will be removed in the next breaking
release."""

import contextlib
import datetime
import signal
import time
from typing import Any, Generator, List


def date_range(
    from_date: datetime.date,
    to_date: datetime.date,
) -> List[datetime.date]:
    """Returns a list of dates between from_date and to_date (inclusive)."""
    delta = to_date - from_date
    assert delta.days >= 0, "from_date must be before to_date"
    return [
        from_date + datetime.timedelta(days=i) for i in range(delta.days + 1)
    ]


@contextlib.contextmanager
def ensure_section_duration(duration: float) -> Generator[None, None, None]:
    """Make sure that the duration of the section is at least the given duration.

    Usage example - do one measurement every 6 seconds:

    ```python
    with ensure_section_duration(6):
        do_measurement()
    ```
    """

    start_time = time.time()
    yield
    remaining_loop_time = start_time + duration - time.time()
    if remaining_loop_time > 0:
        time.sleep(remaining_loop_time)


def set_alarm(timeout: int, label: str) -> None:
    """Set an alarm that will raise a `TimeoutError` after
    `timeout` seconds. The message will be formatted as
    `{label} took too long (timed out after {timeout} seconds)`."""
    def alarm_handler(*args: Any) -> None:
        raise TimeoutError(
            f"{label} took too long (timed out after {timeout} seconds)"
        )

    signal.signal(signal.SIGALRM, alarm_handler)
    signal.alarm(timeout)


def clear_alarm() -> None:
    """Clear the alarm set by `set_alarm`."""

    signal.alarm(0)



"""Context managers for common tasks.

Implements: `ensure_section_duration`, `set_alarm`, `clear_alarm`."""

from typing import Generator
import contextlib
import warnings
from . import timing


@contextlib.contextmanager
def ensure_section_duration(duration: float) -> Generator[None, None, None]:
    """Make sure that the duration of the section is at least the given duration.

    Usage example - do one measurement every 6 seconds:

    ```python
    with ensure_section_duration(6):
        do_measurement()
    ```
    """

    warnings.warn(
        "`context.ensure_section_duration` is deprecated, " +
        "use `timing.ensure_section_duration` instead",
        DeprecationWarning,
    )
    with timing.ensure_section_duration(duration):
        yield


def set_alarm(timeout: int, label: str) -> None:
    """Set an alarm that will raise a `TimeoutError` after
    `timeout` seconds. The message will be formatted as
    `{label} took too long (timed out after {timeout} seconds)`."""

    warnings.warn(
        "`context.set_alarm` is deprecated, " +
        "use `timing.set_alarm` instead",
        DeprecationWarning,
    )
    timing.set_alarm(timeout, label)


def clear_alarm() -> None:
    """Clear the alarm set by `set_alarm`."""

    warnings.warn(
        "`context.clear_alarm` is deprecated, " +
        "use `timing.clear_alarm` instead",
        DeprecationWarning,
    )
    timing.clear_alarm()
